from pathlib import Path
from typing import Any, Annotated
import typer
from zyjared_color import bold
import shutil
import os
import pyperclip as pc
from ...helpers.config import get_config, save_config
from ...helpers.log import log_run, log_title
from ...helpers.command import command, commands

app = typer.Typer()

_CONFIG_PATH = Path.home() / '.zycli.toml'
ENV_DIR = '.venv'
CONFIG_KEYS = {
    'ls': 'list',
}


@app.callback()
def callback():
    """
    Manage venv.
    """


def _config(command: str):
    config = get_config(
        _CONFIG_PATH,
        cliname='venv'
    ) or {}

    if CONFIG_KEYS[command] not in config:
        return None

    return config[CONFIG_KEYS[command]]


def _save(command: str, *, data: Any):
    config = get_config(_CONFIG_PATH, ensure_exists=True)

    if 'venv' not in config:
        config['venv'] = {}

    config['venv'][CONFIG_KEYS[command]] = data

    return save_config(_CONFIG_PATH, config=config)

# region path


def _show():
    return {
        "dataPath": _CONFIG_PATH
    }


@app.command()
def show():
    """
    Show some information.
    """
    log_run(
        _show,
        cliname="venv",
        result_alias="Path",
    )


# region ls


def _ls():
    venvs = _config('ls')
    if venvs is None:
        return []

    res = {}
    for v in venvs:
        res[v['alias']] = {
            'path': v['path'],
            'activate': (Path(os.path.relpath(v['path'], os.getcwd())) / 'Scripts' / 'activate').as_posix()
        }

    return res


@app.command()
def ls():
    """
    List all venvs.
    """
    log_run(
        _ls,
        cliname="venv",
        result_alias="Venvs",
    )

# region add


def _add(alias: str, path: str, *, skip_venv: bool = False, env_dir: str = ENV_DIR):
    venvs = _config('ls')
    if venvs is None:
        venvs = []

    if any(v['alias'] == alias for v in venvs):
        return {"error": f"{bold(alias)} already exists."}

    try:
        if skip_venv:
            _venv_path = Path(path)
        else:
            _venv_path = Path(path) / env_dir
            if _venv_path.exists():
                return {"error": f"{path} already exists."}
            command(
                f'cd {Path(
                    path
                ).absolute().as_posix()} && python -m venv {env_dir}',
                silent=False,
                observe=False
            )

        path = _venv_path.absolute().as_posix()
        venvs.append(
            {"alias": alias, "path": path}
        )

        return {
            "code": _save('ls', data=venvs),
            "alias": alias,
            "path": path,
        }
    except Exception as e:
        return {"error": str(e)}


@app.command()
def add(
    alias: Annotated[
        str,
        typer.Argument(
            help="Specify the venv alias.",
        )
    ],
    path: Annotated[
        str,
        typer.Argument(
            help="Specify the venv path.",
        )
    ],
    skip_venv: Annotated[
        bool,
        typer.Option(
            "--skip",
            show_default=False,
            help="Skip creating venv.",
        )
    ] = False,
    env_dir: Annotated[
        str,
        typer.Option(
            '--dir',
            help="Specify the venv directory name.",
        )
    ] = ENV_DIR
):
    """
    Add a venv.
    """
    log_title(cliname="venv add", tip="执行中...\n")
    log_run(
        lambda: _add(path=path, alias=alias,
                     skip_venv=skip_venv, env_dir=env_dir),
        cliname="venv",
        result_alias="Added",
    )

# region remove


def _remove(alias: str):
    venvs = _config('ls')
    if venvs is None:
        return {"error": "No venvs found."}

    _venvs = []
    _removed = None
    for v in venvs:
        if v['alias'] == alias:
            _removed = v
        else:
            _venvs.append(v)

    if _removed is None:
        return {"error": f"Venv {bold(alias)} not found."}

    if Path(_removed['path']).exists():
        shutil.rmtree(_removed['path'])

    return {
        "code": _save('ls', data=_venvs),
        "alias": _removed['alias'],
        "path": _removed['path'],
    }


@app.command()
def remove(
    alias: Annotated[
        str,
        typer.Argument(
            help="Specify the venv alias.",
        )
    ]
):
    """
    Remove a venv.
    """
    log_run(
        lambda: _remove(alias),
        cliname="venv",
        result_alias="Removed",
    )

# region activate


def _activate(alias: str):
    """
    仅 windows
    """
    venvs = _config('ls')
    if venvs is None:
        return {"error": "No venvs found."}

    target = None
    for v in venvs:
        if v['alias'] == alias:
            target = v
            break

    if target is None:
        return {"error": f"{bold(alias)} not found."}

    try:
        _run = (Path(os.path.relpath(
            target['path'], os.getcwd())) / 'Scripts' / 'activate').as_posix()
    except Exception as e:
        _cur = Path.cwd().absolute().as_posix()
        _dir = Path(target['path']).absolute().as_posix()
        _run = f"cd {_dir} && Scripts/activate && cd {_cur}"

    pc.copy(_run)

    return {
        "alias": target['alias'],
        "path": target['path'],
        "activate": f'{_run}',
        "status": f'{bold('Copied!').green()}'
    }


@app.command()
def activate(
    alias: Annotated[
        str,
        typer.Argument(
            help="Specify the venv alias.",
        )
    ]
):
    """
    Activate a venv.
    """
    log_run(
        lambda: _activate(alias),
        cliname="venv",
        result_alias="Activated",
    )


@app.command()
def run(
    alias: Annotated[
        str,
        typer.Argument(
            help="Specify the venv alias.",
        )
    ]
):
    """
    Alias for activate.
    """
    log_run(
        lambda: _activate(alias),
        cliname="venv",
        result_alias="Run",
    )


# region clean

def _clean():
    venvs = _config('ls')
    if venvs is None:
        return {"error": "No venvs found."}

    removed = []
    dirs_exits = []
    for v in venvs:
        if (Path(v['path']) / 'Scripts' / 'activate').exists():
            dirs_exits.append(v)
        else:
            removed.append(f'{bold(v["alias"])}: {v["path"]}')

    _save('ls', data=removed)
    code = _save('ls', data=dirs_exits)

    return {"code": code, "count": len(removed), "Removed": removed}


@app.command()
def clean():
    """
    Clean up venvs that don't have activate script.
    """
    log_run(
        lambda: _clean(),
        cliname="venv",
        result_alias="Cleaned",
    )
